@@ -1,8 +1,4 @@
 #include "pyobjc.h"
-/* XXX: Introduce OC_BuiltinPythonDate to support
- *      secure coding for builtin classes.
- */
-
 NS_ASSUME_NONNULL_BEGIN
 
 static bool
@@ -163,7 +159,7 @@ is_builtin_datetime(PyObject* object)
      * 3. other (uses PyObjC_encodeWithCoder)
      *
      * As with other classes: non-keyed archives will roundtrip to
-     * Cocoa clases for the exact types datetime.date and datetime.datetime.
+     * Cocoa classes for the exact types datetime.date and datetime.datetime.
      *
      * XXX: Add code to encode the timezone as well.
      */
@@ -181,8 +177,9 @@ is_builtin_datetime(PyObject* object)
             PyObjC_BEGIN_WITH_GIL
                 PyObject* tzinfo = PyObject_GetAttrString(value, "tzinfo");
                 if (tzinfo != NULL && tzinfo != Py_None) {
-                    if (depythonify_python_object(tzinfo, &c_info)
-                        == -1) { // LCOV_BR_EXCL_LINE
+                    if (depythonify_python_object( // LCOV_BR_EXCL_LINE
+                            tzinfo, &c_info)
+                        == -1) {
                         // LCOV_EXCL_START
                         Py_DECREF(tzinfo);
                         PyObjC_GIL_FORWARD_EXC();
