@@ -383,8 +383,8 @@ void
 PyObjCErr_ToObjCWithGILState(PyGILState_STATE* _Nonnull state)
 {
     NSException* exc = PyObjCErr_AsExc();
-    if (exc == nil)
-        PyObjCErr_InternalError(); // LCOV_BR_EXCL_LINE
+    if (exc == nil)                // LCOV_BR_EXCL_LINE
+        PyObjCErr_InternalError(); // LCOV_EXCL_LINE
 
     if (state) {
         PyGILState_Release(*state);
@@ -762,6 +762,7 @@ PyObjC_PythonToCArray(BOOL writable, BOOL exactSize, const char* elementType,
                 /* Ensure that the expected semantics still work
                  * when the passed in buffer is read-only
                  */
+                PyErr_Clear();
                 PyObject* byte_array;
 
                 if (PyObject_GetBuffer(pythonList, view, PyBUF_CONTIG_RO) == -1) {
@@ -1075,7 +1076,7 @@ PyObjC_PythonToCArray(BOOL writable, BOOL exactSize, const char* elementType,
     }
 }
 
-PyObject* _Nullable PyObjC_CArrayToPython(const char* elementType, void* array,
+PyObject* _Nullable PyObjC_CArrayToPython(const char* elementType, const void* array,
                                           Py_ssize_t size)
 {
     PyObject*  result;
@@ -1142,9 +1143,6 @@ PyObjC_IsPythonKeyword(const char* word)
     return 0;
 }
 
-/*
- * XXX: This function may or may not raise an exception on error.
- */
 int
 PyObjCRT_SimplifySignature(const char* signature, char* buf, size_t buflen)
 {
@@ -1167,6 +1165,7 @@ PyObjCRT_SimplifySignature(const char* signature, char* buf, size_t buflen)
         end++;
 
         if ((size_t)(end - cur) > buflen) {
+            PyErr_SetString(PyObjCExc_Error, "signature too long");
             return -1;
         }
 
@@ -1179,7 +1178,7 @@ PyObjCRT_SimplifySignature(const char* signature, char* buf, size_t buflen)
     return 0;
 }
 
-PyObject* _Nullable PyObjC_CArrayToPython2(const char* elementType, void* array,
+PyObject* _Nullable PyObjC_CArrayToPython2(const char* elementType, const void* array,
                                            Py_ssize_t size, bool alreadyRetained,
                                            bool alreadyCFRetained)
 {
@@ -1353,11 +1352,13 @@ PyObjCRT_SignaturesEqual(const char* sig1, const char* sig2)
      */
     r = PyObjCRT_SimplifySignature(sig1, buf1, sizeof(buf1));
     if (r == -1) {
+        PyErr_Clear();
         return 0;
     }
 
     r = PyObjCRT_SimplifySignature(sig2, buf2, sizeof(buf2));
     if (r == -1) {
+        PyErr_Clear();
         return 0;
     }
     return strcmp(buf1, buf2) == 0;
