@@ -3,9 +3,10 @@
 
 /*
  * Central include file for PyObjC.
+ *
  */
 
-#define OBJC_VERSION "8.4.1"
+#define OBJC_VERSION "9.0.1"
 
 #define PY_SSIZE_T_CLEAN
 #include <Python.h>
@@ -118,7 +119,6 @@
 
 #import "selector.h"
 
-#import "block-as-imp.h"
 #import "libffi_extra.h"
 #import "libffi_support.h"
 
@@ -138,6 +138,7 @@
 #import "instance-var.h"
 #import "ivar-accessor.h"
 #import "memview.h"
+#import "meth-func.h"
 #import "method-accessor.h"
 #import "method-imp.h"
 #import "objc-NULL.h"
