@@ -1,9 +1,10 @@
 #ifndef PyObjC_WEAKREF_H
 #define PyObjC_WEAKREF_H
 
-#if PyObjC_BUILD_RELEASE >= 1007
+NS_ASSUME_NONNULL_BEGIN
+
 extern PyTypeObject PyObjCWeakRef_Type;
-#endif /* PyObjC_BUILD_RELEASE >= 1007 */
+
+NS_ASSUME_NONNULL_END
 
 #endif /* PyObjC_WEAKREF_H */
-
