@@ -1,7 +1,7 @@
 import sys
 
 import objc
-from PyObjCTools.TestSupport import TestCase, main
+from PyObjCTools.TestSupport import TestCase
 
 # Most useful systems will at least have 'NSObject'.
 # NSObject = objc.lookUpClass('NSObject')
@@ -11,7 +11,7 @@ from PyObjCTools.TestSupport import TestCase, main
 BaseName = "NSAttributedString"
 BaseClass = objc.lookUpClass(BaseName)
 
-if sys.maxsize >= 2 ** 32:
+if sys.maxsize >= 2**32:
     # -poseAsClass: is not supported in 64-bit mode (the functionality is
     # not present in the 64-bit runtime and will never be because it
     # conflicts with new functionality such as non-fragile class layouts)
@@ -40,7 +40,3 @@ else:
             self.assertNotEqual(BaseClass.__name__, BaseName)
             self.assertEqual(PoseClass.__name__, BaseName)
             del obj
-
-
-if __name__ == "__main__":
-    main()
